@@ -5,7 +5,8 @@ import torch.optim as optim
 
 import lpd.utils.torch_utils as tu
 from lpd.extensions.custom_layers import TransformerEncoderStack, Attention, MatMul2D
-from lpd.callbacks import EpochEndStats, ModelCheckPoint, Tensorboard, EarlyStopping
+import lpd.callbacks as cbs 
+from lpd.callbacks import EpochEndStats, ModelCheckPoint, Tensorboard, EarlyStopping, SchedulerStep
 from lpd.extensions.custom_metrics import binary_accuracy_with_logits
 from lpd.trainer import Trainer
 from lpd.extensions.custom_schedulers import DoNothingToLR
@@ -39,14 +40,14 @@ class TestModel(nn.Module):
         # x3   : some2-Input        (batch, 1)
 
         x1_emb = self.embedding_layer(x1)                                                     # (batch, num_elements, emb_size)
-        member_transformed = self.transformer_encoder(x1_emb)                                 # (batch, num_elements, emb_size)
+        x1_emb_transformed = self.transformer_encoder(x1_emb)                                 # (batch, num_elements, emb_size)
         
         x3_emb = self.embedding_layer(x3)                                                     # (batch, emb_size)
         x3_emb_unsqueesed = x3_emb.unsqueeze(1)                                               # (batch, 1, emb_size)
 
         x1_with_x3_reduced = self.external_query_attention(q=x3_emb_unsqueesed, 
-                                                           k=member_transformed, 
-                                                           v=member_transformed)              # (batch, 1, emb_size)
+                                                           k=x1_emb_transformed, 
+                                                           v=x1_emb_transformed)              # (batch, 1, emb_size)
         
 
         x1_with_x3_residual = self.norm(x1_with_x3_reduced + x3_emb_unsqueesed)     		  # (batch, 1, emb_size)
@@ -71,24 +72,24 @@ def get_trainer(config,
 
     model = TestModel(config, num_embeddings).to(device)
    
-    optimizer = optim.SGD(params=model.parameters(), lr=0.1, momentum=0.9)
+    optimizer = optim.SGD(params=model.parameters(), lr=config.LEARNING_RATE, momentum=0.9)
     # optimizer = optim.Adam(params=model.parameters(), lr=config.LEARNING_RATE)
-    # optimizer = optim.Adam(params=model.parameters(), lr=0.1)
 
     # scheduler = DoNothingToLR(optimizer=optimizer)
-    scheduler = optim.lr_scheduler.ReduceLROnPlateau(optimizer, mode='min', patience=5, verbose=True)
-    # scheduler = optim.lr_scheduler.StepLR(optimizer=optimizer, gamma=config.STEP_LR_GAMMA, step_size=config.STEP_LR_STEP_SIZE) 
+    scheduler = optim.lr_scheduler.ReduceLROnPlateau(optimizer, mode='min', patience=config.EARLY_STOPPING_PATIENCE // 2, verbose=True) # needs SchedulerStep callback WITH scheduler_parameters_func
+    # scheduler = optim.lr_scheduler.StepLR(optimizer=optimizer, gamma=config.STEP_LR_GAMMA, step_size=config.STEP_LR_STEP_SIZE) # needs SchedulerStep callback WITHOUT scheduler_parameters_func
     
     loss_func = nn.BCEWithLogitsLoss().to(device)
 
     metric_name_to_func = {"acc":binary_accuracy_with_logits}
 
-    cbs = [
-            ModelCheckPoint(checkpoint_dir, checkpoint_file_name, monitor='val_loss', save_best_only=True), 
-            Tensorboard(summary_writer_dir=summary_writer_dir),
-            EarlyStopping(patience=config.PATIENCE, monitor='val_loss'),
-            EpochEndStats() # BETTER TO PUT IT LAST (MAKES BETTER SENSE IN THE LOG PRINTS)
-           ]
+    callbacks = [   
+                    SchedulerStep(scheduler_parameters_func=lambda trainer: trainer.val_stats.get_loss()),
+                    ModelCheckPoint(checkpoint_dir, checkpoint_file_name, monitor='val_loss', save_best_only=True, round_values_on_print_to=7), 
+                    Tensorboard(summary_writer_dir=summary_writer_dir),
+                    EarlyStopping(patience=config.EARLY_STOPPING_PATIENCE, monitor='val_loss'),
+                    EpochEndStats(cb_phase=cbs.CB_ON_EPOCH_END, round_values_on_print_to=7) # BETTER TO PUT IT LAST (MAKES BETTER SENSE IN THE LOG PRINTS)
+                ]
 
     trainer = Trainer(model=model, 
                       device=device, 
@@ -101,6 +102,5 @@ def get_trainer(config,
                       train_steps=train_steps,
                       val_steps=val_steps,
                       num_epochs=num_epochs,
-                      callbacks=cbs,
-                      print_round_values_to=5)
+                      callbacks=callbacks)
     return trainer
